'use strict';

var _require = require('./replace-selectors'),
    replaceSelectors = _require.replaceSelectors;

var _require2 = require('./utils'),
    getNodes = _require2.getNodes,
    markNodes = _require2.markNodes,
    getSelectors = _require2.getSelectors,
    getStylesAndAllSelectors = _require2.getStylesAndAllSelectors;

function createSerializer(styleSheet, classNameReplacer) {

  return { test: function (val) {
      return val && !val.serializedWithJestGlamorReact && (val.$$typeof === Symbol.for('react.test.json') || val instanceof HTMLElement && !isBeingSerialized(val));
    }, print: function (val, printer) {
      var nodes = getNodes(val);
      markNodes(nodes);
      var selectors = getSelectors(nodes);

      var _getStylesAndAllSelec = getStylesAndAllSelectors(selectors, styleSheet),
          styles = _getStylesAndAllSelec.styles,
          allSelectors = _getStylesAndAllSelec.allSelectors;

      var printedVal = printer(val);
      if (styles) {
        return replaceSelectors(allSelectors, styles, printedVal, classNameReplacer);
      } else {
        return printedVal;
      }
    } };
}

function isBeingSerialized(node) {
  var currentNode = node;

  while (currentNode) {
    if (currentNode.serializedWithJestGlamorReact) {
      return true;
    }
    currentNode = currentNode.parentNode;
  }
  return false;
}

// doing this to make it easier for users to mock things
// like switching between development mode and whatnot.
var getGlamorStyleSheet = function () {
  return require('glamor').styleSheet;
};
var glamorSerializer = createSerializer(getGlamorStyleSheet);
createSerializer.test = glamorSerializer.test;
createSerializer.print = glamorSerializer.print;

module.exports = createSerializer;